@@ -3,11 +3,10 @@ import math
 from torch import nn
 from product_key_memory.evonorm import EvoNorm1D
 
-
 def init_(t, dim = None):
     dim = dim if dim is not None else t.shape[-1]
     std = 1. / math.sqrt(dim)
-    return t.normal_(0, std)
+    return nn.init.normal_(t, mean=0, std=std)
 
 def expand_dim(t, dim, k, unsqueeze = False):
     if unsqueeze:
@@ -16,23 +15,44 @@ def expand_dim(t, dim, k, unsqueeze = False):
     expand_shape[dim] = k
     return t.expand(*expand_shape)
 
+def list_subtract(l, r):
+    return [el for el in l if el not in set(r)]
+
 def fetch_pkm_value_parameters(module):
     params = []
     for m in module.modules():
         if isinstance(m, PKM):
             params.append(m.values.weight)
-    return params
+    rest = list_subtract(module.parameters(), params)
+    return params, rest
 
-class MergeDims(nn.Module):
+def fetch_parameters(module, pkm_learning_rate = 1e-2):
+    pkm_params, rest = fetch_pkm_value_parameters(module)
+    return [{'params': rest}, {'params': pkm_params, 'lr': pkm_learning_rate}]
+
+class MaskedBatchNorm1D(nn.Module):
     def __init__(self, fn):
         super().__init__()
         self.fn = fn
 
-    def forward(self, x):
+    def forward(self, x, mask = None):
+        b, t, d = x.shape
+        has_mask = mask is not None
+
+        if has_mask:
+            initial_x = x
+            mask = mask.unsqueeze(-1)
+            x = x.masked_select(mask)
+
         shape = x.shape
-        x = x.reshape(-1, shape[-1])
+        x = x.reshape(-1, d)
         x = self.fn(x)
-        return x.reshape(*shape)
+        x = x.reshape(*shape)
+
+        if has_mask:
+            x = initial_x.masked_scatter(mask, x)
+
+        return x
 
 class PKM(nn.Module):
     def __init__(self, dim, heads = 4, num_keys = 128, topk = 32, dim_head = 256, input_dropout = 0., query_dropout = 0., value_dropout = 0., use_evonorm = False):
@@ -44,22 +64,23 @@ class PKM(nn.Module):
 
         dim_query = dim_head * heads
         self.to_queries = nn.Linear(dim, dim_query, bias = False)
-        self.norm = MergeDims(nn.BatchNorm1d(dim_query)) if not use_evonorm else EvoNorm1D(dim_query)
+        self.norm = MaskedBatchNorm1D(nn.BatchNorm1d(dim_query)) if not use_evonorm else EvoNorm1D(dim_query)
 
-        keys = init_(torch.randn(heads, num_keys, 2, dim_head // 2))
-        self.keys = nn.Parameter(keys)
+        self.keys = nn.Parameter(torch.zeros(heads, num_keys, 2, dim_head // 2))
         self.values = nn.EmbeddingBag(num_keys ** 2, dim, mode='sum')
+        init_(self.keys)
+        init_(self.values.weight)
 
         self.input_dropout = nn.Dropout(input_dropout)
         self.query_dropout = nn.Dropout(query_dropout)
         self.value_dropout = nn.Dropout(value_dropout)
 
-    def forward(self, x, **kwargs):
+    def forward(self, x, input_mask = None, **kwargs):
         b, t, e, h = *x.shape, self.heads
         x = self.input_dropout(x)
 
         queries = self.to_queries(x)
-        queries = self.norm(queries)
+        queries = self.norm(queries, mask = input_mask)
         queries = self.query_dropout(queries)
 
         queries = queries.chunk(2, dim=-1)
